<?php

namespace Drupal\static_node\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Pager\PagerManagerInterface;

/**
 * Controller for the static files management page.
 */
class StaticFileManagementController extends ControllerBase {

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $pathAliasManager;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;
  
  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;
  
  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;
  
  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * Constructs a StaticFileManagementController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\path_alias\AliasManagerInterface $path_alias_manager
   *   The path alias manager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pager_manager
   *   The pager manager.
   */
  public function __construct(
    Connection $database,
    EntityTypeManagerInterface $entity_type_manager,
    AliasManagerInterface $path_alias_manager,
    FileUrlGeneratorInterface $file_url_generator,
    DateFormatterInterface $date_formatter,
    RequestStack $request_stack,
    PagerManagerInterface $pager_manager
  ) {
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
    $this->pathAliasManager = $path_alias_manager;
    $this->fileUrlGenerator = $file_url_generator;
    $this->dateFormatter = $date_formatter;
    $this->requestStack = $request_stack;
    $this->pagerManager = $pager_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('entity_type.manager'),
      $container->get('path_alias.manager'),
      $container->get('file_url_generator'),
      $container->get('date.formatter'),
      $container->get('request_stack'),
      $container->get('pager.manager')
    );
  }

  /**
   * Overview page for static files.
   *
   * @return array
   *   A render array for the overview page.
   */
  public function overview() {
    // Build the file table
    $header = [
      ['data' => $this->t('Static File'), 'field' => 'filename', 'sort' => 'asc'],
      ['data' => $this->t('Type')],
      ['data' => $this->t('Source')],
      ['data' => $this->t('Created'), 'field' => 'created', 'sort' => 'desc'],
      ['data' => $this->t('Operations')],
    ];

    $query = $this->entityTypeManager->getStorage('file')->getQuery()
      ->condition('uri', 'public://' . $this->getStaticDirectoryName() . '/%', 'LIKE')
      ->sort('created', 'DESC')
      ->accessCheck(FALSE); // Disable access checking for system operations
    $pager = $query->pager(20);
    $fids = $query->execute();
    $files = $this->entityTypeManager->getStorage('file')->loadMultiple($fids);

    $rows = [];

    foreach ($files as $file) {
      $filepath = $file->getFileUri();
      $file_url = $this->fileUrlGenerator->generateAbsoluteString($filepath);
      $basename = basename($filepath);
      
      // Determine the file type based on the path or other attributes
      if (strpos($basename, 'index.html') !== FALSE || strpos($filepath, 'index.html') !== FALSE) {
        $type = $this->t('Homepage');
      } 
      else {
        $type = $this->t('Node');
      }
      
      // Try to find the source node if available
      $source = '';
      if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty()) {
        $nid = $file->field_static_node->target_id;
        try {
          $node = $this->entityTypeManager->getStorage('node')->load($nid);
          if ($node) {
            $source = [
              '#type' => 'link',
              '#title' => $node->getTitle(),
              '#url' => $node->toUrl(),
            ];
          }
        }
        catch (\Exception $e) {
          $this->getLogger('static_node')->error('Error loading source node: @error', ['@error' => $e->getMessage()]);
        }
      }
      
      $operations = [];
      
      // Add a link to view the static file
      $operations['view'] = [
        '#type' => 'link',
        '#title' => $this->t('View'),
        '#url' => Url::fromUri($file_url),
        '#attributes' => ['class' => ['button', 'button--small'], 'target' => '_blank'],
      ];
      
      // Add a delete operation if the user has permission
      if ($this->currentUser->hasPermission('delete static node')) {
        $operations['delete'] = [
          '#type' => 'link',
          '#title' => $this->t('Delete'),
          '#url' => Url::fromRoute('static_node.delete_static_file', ['file' => $file->id()]),
          '#attributes' => ['class' => ['button', 'button--small', 'button--danger']],
        ];
      }

      // If we have a source node and user has regenerate permission
      if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty() && 
          $this->currentUser->hasPermission('generate static node')) {
        $nid = $file->field_static_node->target_id;
        $operations['regenerate'] = [
          '#type' => 'link',
          '#title' => $this->t('Regenerate'),
          '#url' => Url::fromRoute('static_node.regenerate_static_file', ['node' => $nid]),
          '#attributes' => ['class' => ['button', 'button--small']],
        ];
      }

      $row = [
        'filename' => [
          '#type' => 'link',
          '#title' => $basename,
          '#url' => Url::fromUri($file_url),
          '#attributes' => ['target' => '_blank'],
        ],
        'type' => $type,
        'source' => $source,
        'created' => $this->dateFormatter->format($file->getCreatedTime(), 'short'),
        'operations' => [
          'data' => $operations,
        ],
      ];

      $rows[] = $row;
    }

    $build = [];
    
    // Add explanatory text
    $build['explanation'] = [
      '#markup' => '<p>' . $this->t('This page lists all static HTML files that have been generated. You can view, regenerate, or delete these files.') . '</p>',
    ];
    
    // Add action buttons
    $build['actions'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['action-buttons', 'container-inline', 'margin-bottom']],
    ];
    
    // Homepage generation buttons
    $build['actions']['generate_homepage'] = [
      '#type' => 'link',
      '#title' => $this->t('Generate static homepage'),
      '#url' => Url::fromRoute('static_node.generate_homepage'),
      '#attributes' => ['class' => ['button', 'button-action']],
    ];
    
    $build['actions']['delete_homepage'] = [
      '#type' => 'link',
      '#title' => $this->t('Delete static homepage'),
      '#url' => Url::fromRoute('static_node.delete_homepage'),
      '#attributes' => ['class' => ['button']],
    ];
    
    // Add custom path generation button
    $build['actions']['generate_custom_path'] = [
      '#type' => 'link',
      '#title' => $this->t('Generate static page for custom path'),
      '#url' => Url::fromRoute('static_node.generate_custom_path'),
      '#attributes' => ['class' => ['button', 'button-action']],
    ];
    
    // Table with files
    $build['files_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No static files have been generated yet.'),
      '#attributes' => ['class' => ['static-files-table']],
    ];
    
    // Add the pager
    $build['pager'] = [
      '#type' => 'pager',
    ];
    
    return $build;
  }

  /**
   * Format bytes to human-readable file size.
   *
   * @param int $bytes
   *   File size in bytes.
   * @param int $precision
   *   Number of decimal places.
   *
   * @return string
   *   Human-readable file size.
   */
  protected function formatBytes($bytes, $precision = 2) {
    $units = ['B', 'KB', 'MB', 'GB', 'TB'];
    
    $bytes = max($bytes, 0);
    $pow = floor(($bytes ? log($bytes) : 0) / log(1024));
    $pow = min($pow, count($units) - 1);
    
    $bytes /= (1 << (10 * $pow));
    
    return round($bytes, $precision) . ' ' . $units[$pow];
  }
}
