<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\file\FileInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;

/**
 * Provides a confirmation form for deleting a static file.
 */
class StaticFileDeleteFileForm extends ConfirmFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The file entity.
   *
   * @var \Drupal\file\FileInterface
   */
  protected $file;

  /**
   * Constructs a StaticFileDeleteFileForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileSystemInterface $file_system,
    MessengerInterface $messenger
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file_system'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_delete_file_confirm_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete the static file %filename?', ['%filename' => $this->file->getFilename()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return Url::fromRoute('static_node.files_management');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    $description = $this->t('This will permanently delete the static HTML file.');
    
    // If the file is associated with a node, add additional information
    if ($this->file->hasField('field_static_node') && !$this->file->field_static_node->isEmpty()) {
      $nid = $this->file->field_static_node->target_id;
      try {
        $node = $this->entityTypeManager->getStorage('node')->load($nid);
        if ($node) {
          $description .= ' ' . $this->t('This file is associated with the node: %title.', ['%title' => $node->getTitle()]);
        }
      }
      catch (\Exception $e) {
        // If there's an error loading the node, don't add additional information
      }
    }
    
    return $description;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelText() {
    return $this->t('Cancel');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, FileInterface $file = NULL) {
    $this->file = $file;
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    try {
      // Delete the file entity
      $filename = $this->file->getFilename();
      $this->file->delete();
      $this->messenger->addStatus($this->t('Static file %filename has been deleted.', ['%filename' => $filename]));
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('Error deleting static file: @error', ['@error' => $e->getMessage()]));
    }
    
    // Redirect to the files management page
    $form_state->setRedirectUrl(Url::fromRoute('static_node.files_management'));
  }

}